@@ -7,6 +7,7 @@ import base64
 import json
 import subprocess
 from pathlib import Path
+from typing import Any
 
 import click
 from fastmcp import FastMCP
@@ -155,6 +156,7 @@ async def start_mcp_proxy(
     import asyncio
     import logging
     import os
+    import signal
     import sys
 
     from .utils.logging import find_free_port
@@ -440,12 +442,30 @@ async def start_mcp_proxy(
                     log_design.warning(f"Traceback: {traceback.format_exc()}")  # noqa: G004
                 await asyncio.sleep(1)
 
+    # Import contextlib here so it's available in the finally block
+    import contextlib
+
     # CRITICAL: Create proxy AFTER all logging setup to prevent it from resetting logging config
     # This is important because FastMCP might initialize loggers during creation
     proxy = create_proxy_server(
         directory, image_name, no_reload, full_reload, verbose, docker_args or [], interactive
     )
 
+    # Set up signal handlers for graceful shutdown
+    shutdown_event = asyncio.Event()
+
+    def signal_handler(signum: int, frame: Any) -> None:
+        """Handle signals by setting shutdown event."""
+        design.info(f"\n📡 Received signal {signum}, shutting down gracefully...")
+        shutdown_event.set()
+
+    # Register signal handlers - SIGINT is available on all platforms
+    signal.signal(signal.SIGINT, signal_handler)
+
+    # SIGTERM is not available on Windows
+    if hasattr(signal, "SIGTERM"):
+        signal.signal(signal.SIGTERM, signal_handler)
+
     # One more attempt to suppress the FastMCP server log
     if not verbose:
         # Re-apply the filter in case new handlers were created
@@ -479,6 +499,47 @@ async def start_mcp_proxy(
             for handler in logger.handlers:
                 handler.addFilter(block_filter)
 
+    # Track if container has been stopped to avoid duplicate stops
+    container_stopped = False
+
+    # Function to stop the container gracefully
+    async def stop_container() -> None:
+        """Stop the Docker container gracefully with SIGTERM, wait 30s, then SIGKILL if needed."""
+        nonlocal container_stopped
+        if container_stopped:
+            return  # Already stopped, don't do it again
+
+        try:
+            # Check if container exists
+            check_result = await asyncio.create_subprocess_exec(
+                "docker",
+                "ps",
+                "--format",
+                "{{.Names}}",
+                "--filter",
+                f"name={container_name}",
+                stdout=asyncio.subprocess.PIPE,
+                stderr=asyncio.subprocess.DEVNULL,
+            )
+            stdout, _ = await check_result.communicate()
+
+            if container_name in stdout.decode():
+                design.info("🛑 Stopping container gracefully...")
+                # Stop with 30 second timeout before SIGKILL
+                stop_result = await asyncio.create_subprocess_exec(
+                    "docker",
+                    "stop",
+                    "--time=30",
+                    container_name,
+                    stdout=asyncio.subprocess.DEVNULL,
+                    stderr=asyncio.subprocess.DEVNULL,
+                )
+                await stop_result.communicate()
+                design.success("✅ Container stopped successfully")
+                container_stopped = True
+        except Exception as e:
+            design.warning(f"Failed to stop container: {e}")
+
     try:
         # Start Docker logs streaming if enabled
         log_task = None
@@ -530,6 +591,9 @@ async def start_mcp_proxy(
     except KeyboardInterrupt:
         design.info("\n👋 Shutting down...")
 
+        # Stop the container before showing next steps
+        await stop_container()
+
         # Show next steps tutorial
         if not interactive:  # Only show if not in interactive mode
             design.section_title("Next Steps")
@@ -565,6 +629,9 @@ async def start_mcp_proxy(
             except asyncio.CancelledError:
                 contextlib.suppress(asyncio.CancelledError)
 
+        # Always try to stop container on exit
+        await stop_container()
+
 
 def run_mcp_dev_server(
     directory: str = ".",
