@@ -173,7 +173,7 @@ class BaseHUDClient(AgentMCPClient):
                 logger.warning("Failed to close auto-created trace: %s", e)
             finally:
                 self._auto_trace_cm = None
-        
+
         # Disconnect from server
         if self._initialized:
             await self._disconnect()
@@ -286,6 +286,8 @@ class BaseHUDClient(AgentMCPClient):
                 logger.info("📡 Telemetry data fetched:")
                 if "live_url" in telemetry_data:
                     logger.info("   🖥️  Live URL: %s", telemetry_data["live_url"])
+                if "vnc_url" in telemetry_data:
+                    logger.info("   🖥️  VNC URL: %s", telemetry_data["vnc_url"])
                 if "cdp_url" in telemetry_data:
                     logger.info("   🦾  CDP URL: %s", telemetry_data["cdp_url"])
                 if "status" in telemetry_data:
