@@ -14,6 +14,11 @@ Color Palette:
 
 from __future__ import annotations
 
+import logging
+from typing import Any
+
+import questionary
+import typer
 from rich.console import Console
 from rich.panel import Panel
 from rich.table import Table
@@ -28,10 +33,15 @@ DIM = "dim"
 class HUDDesign:
     """Design system for HUD CLI output."""
 
-    def __init__(self) -> None:
-        """Initialize the design system."""
+    def __init__(self, logger: logging.Logger | None = None) -> None:
+        """Initialize the design system.
+
+        Args:
+            logger: Logger to check for log levels. If None, uses the root logger.
+        """
         self._stdout_console = Console(stderr=False)
         self._stderr_console = Console(stderr=True)
+        self._logger = logger or logging.getLogger()
 
     def header(self, title: str, icon: str = "🚀", stderr: bool = True) -> None:
         """Print a header panel with gold border.
@@ -252,6 +262,162 @@ class HUDDesign:
         """Get the stderr console for direct access when needed."""
         return self._stderr_console
 
+    def set_verbose(self, verbose: bool) -> None:
+        """Set the logging level based on verbose flag.
+
+        Args:
+            verbose: If True, show INFO level messages. If False, only show WARNING and above.
+        """
+        if verbose:
+            self._logger.setLevel(logging.INFO)
+        else:
+            self._logger.setLevel(logging.WARNING)
+
+    # Logging-aware methods that check logging levels before printing
+
+    def debug(self, message: str, stderr: bool = True) -> None:
+        """Print a debug message only if DEBUG logging is enabled.
+
+        Args:
+            message: The debug message
+            stderr: If True, output to stderr (default), otherwise stdout
+        """
+        if self._logger.isEnabledFor(logging.DEBUG):
+            self.dim_info("[DEBUG]", message, stderr=stderr)
+
+    def info_log(self, message: str, stderr: bool = True) -> None:
+        """Print an info message only if INFO logging is enabled.
+
+        Args:
+            message: The info message
+            stderr: If True, output to stderr (default), otherwise stdout
+        """
+        if self._logger.isEnabledFor(logging.INFO):
+            self.info(message, stderr=stderr)
+
+    def progress_log(self, message: str, stderr: bool = True) -> None:
+        """Print a progress message only if INFO logging is enabled.
+
+        Args:
+            message: The progress message
+            stderr: If True, output to stderr (default), otherwise stdout
+        """
+        if self._logger.isEnabledFor(logging.INFO):
+            self.progress_message(message, stderr=stderr)
+
+    def success_log(self, message: str, stderr: bool = True) -> None:
+        """Print a success message only if INFO logging is enabled.
+
+        Args:
+            message: The success message
+            stderr: If True, output to stderr (default), otherwise stdout
+        """
+        if self._logger.isEnabledFor(logging.INFO):
+            self.success(message, stderr=stderr)
+
+    def warning_log(self, message: str, stderr: bool = True) -> None:
+        """Print a warning message only if WARNING logging is enabled.
+
+        Args:
+            message: The warning message
+            stderr: If True, output to stderr (default), otherwise stdout
+        """
+        if self._logger.isEnabledFor(logging.WARNING):
+            self.warning(message, stderr=stderr)
+
+    def error_log(self, message: str, stderr: bool = True) -> None:
+        """Print an error message only if ERROR logging is enabled.
+
+        Args:
+            message: The error message
+            stderr: If True, output to stderr (default), otherwise stdout
+        """
+        if self._logger.isEnabledFor(logging.ERROR):
+            self.error(message, stderr=stderr)
+
+    def select(
+        self,
+        message: str,
+        choices: list[str | dict[str, Any]] | list[str],
+        default: str | None = None,
+    ) -> str:
+        """Interactive selection with arrow key navigation.
+
+        Args:
+            message: The prompt message to display
+            choices: List of choices. Can be strings or dicts with 'name' and 'value' keys
+            default: Default selection (matches against choice name/string)
+
+        Returns:
+            The selected choice value
+        """
+        # Convert choices to questionary format
+        q_choices = []
+
+        for choice in choices:
+            if isinstance(choice, dict):
+                name = choice.get("name", str(choice.get("value", "")))
+                value = choice.get("value", name)
+                q_choices.append(questionary.Choice(title=name, value=value))
+            else:
+                q_choices.append(choice)
+
+        result = questionary.select(
+            message,
+            choices=q_choices,
+            instruction="(Use ↑/↓ arrows, Enter to select)",
+        ).ask()
+
+        # If no selection made (Ctrl+C or ESC), exit
+        if result is None:
+            raise typer.Exit(1)
+
+        return result
+
+    def format_tool_call(self, name: str, arguments: dict[str, Any] | None = None) -> str:
+        """Format a tool call in compact HUD style.
+
+        Args:
+            name: Tool name
+            arguments: Tool arguments dictionary
+
+        Returns:
+            Formatted string with Rich markup
+        """
+        import json
+
+        args_str = ""
+        if arguments:
+            try:
+                # Compact JSON representation
+                args_str = json.dumps(arguments, separators=(",", ":"))
+                if len(args_str) > 60:
+                    args_str = args_str[:57] + "..."
+            except (TypeError, ValueError):
+                args_str = str(arguments)[:60]
+
+        return f"[{GOLD}]→[/{GOLD}] [bold]{name}[/bold][{DIM}]({args_str})[/{DIM}]"
+
+    def format_tool_result(self, content: str, is_error: bool = False) -> str:
+        """Format a tool result in compact HUD style.
+
+        Args:
+            content: Result content (will be truncated if too long)
+            is_error: Whether this is an error result
+
+        Returns:
+            Formatted string with Rich markup
+        """
+        # Truncate content if needed
+        if len(content) > 80:
+            content = content[:77] + "..."
+
+        # Format with status using HUD colors
+        if is_error:
+            return f"  [{RED}]✗[/{RED}] [{DIM}]{content}[/{DIM}]"
+        else:
+            return f"  [{GREEN}]✓[/{GREEN}] {content}"
+
 
 # Global design instance for convenience
 design = HUDDesign()
